import typing

from telegrinder.bot.cute_types.base import BaseCute
from telegrinder.bot.dispatch.context import Context
from telegrinder.msgspec_utils import Option
from telegrinder.types.enums import ChatType, DiceEmoji
from telegrinder.types.objects import User

from .abc import ABCRule, Message, MessageRule

T = typing.TypeVar("T", bound=BaseCute)


@typing.runtime_checkable
class HasFromProto(typing.Protocol):
    from_: User | Option[User]


class HasFrom(ABCRule[T]):
    async def check(self, event: T, ctx: Context) -> bool:
        return isinstance(event, HasFromProto) and bool(event.from_)


class HasDice(MessageRule):
    async def check(self, message: Message, ctx: Context) -> bool:
        return bool(message.dice)


class IsReply(MessageRule):
    async def check(self, message: Message, ctx: Context) -> bool:
        return bool(message.reply_to_message)


class IsSticker(MessageRule):
    async def check(self, message: Message, ctx: Context) -> bool:
        return bool(message.sticker)


class IsBot(MessageRule, requires=[HasFrom()]):
    async def check(self, message: Message, ctx: Context) -> bool:
        return message.from_user.is_bot


class IsUser(MessageRule, requires=[HasFrom()]):
    async def check(self, message: Message, ctx: Context) -> bool:
        return not message.from_user.is_bot


class IsPremium(MessageRule, requires=[HasFrom()]):
    async def check(self, message: Message, ctx: Context) -> bool:
        return message.from_user.is_premium.unwrap_or(False)


class IsLanguageCode(MessageRule, requires=[HasFrom()]):
    def __init__(self, lang_codes: str | list[str], /) -> None:
        self.lang_codes = [lang_codes] if isinstance(lang_codes, str) else lang_codes

    async def check(self, message: Message, ctx: Context) -> bool:
        if not message.from_user.language_code:
            return False
        return message.from_user.language_code.unwrap_or_none() in self.lang_codes


class IsForum(MessageRule):
    async def check(self, message: Message, ctx: Context) -> bool:
        return message.chat.is_forum.unwrap_or(False)


class IsUserId(MessageRule, requires=[HasFrom()]):
    def __init__(self, user_ids: int | list[int], /) -> None:
        self.user_ids = [user_ids] if isinstance(user_ids, int) else user_ids

    async def check(self, message: Message, ctx: Context) -> bool:
        return message.from_user.id in self.user_ids


class IsChatId(MessageRule):
    def __init__(self, chat_ids: int | list[int], /) -> None:
        self.chat_ids = [chat_ids] if isinstance(chat_ids, int) else chat_ids

    async def check(self, message: Message, ctx: Context) -> bool:
        return message.chat.id in self.chat_ids


class IsPrivate(MessageRule):
    async def check(self, message: Message, ctx: Context) -> bool:
        return message.chat.type == ChatType.PRIVATE


class IsGroup(MessageRule):
    async def check(self, message: Message, ctx: Context) -> bool:
        return message.chat.type == ChatType.GROUP


class IsSuperGroup(MessageRule):
    async def check(self, message: Message, ctx: Context) -> bool:
        return message.chat.type == ChatType.SUPERGROUP


class IsChat(MessageRule):
    async def check(self, message: Message, ctx: Context) -> bool:
        return message.chat.type in (ChatType.GROUP, ChatType.SUPERGROUP)


class IsDice(MessageRule, requires=[HasDice()]):
    async def check(self, message: Message, ctx: Context) -> bool:
        return message.dice.unwrap().emoji == DiceEmoji.DICE


class IsDartDice(MessageRule, requires=[HasDice()]):
    async def check(self, message: Message, ctx: Context) -> bool:
        return message.dice.unwrap().emoji == DiceEmoji.DART


class IsBasketballDice(MessageRule, requires=[HasDice()]):
    async def check(self, message: Message, ctx: Context) -> bool:
        return message.dice.unwrap().emoji == DiceEmoji.BASKETBALL


class IsFootballDice(MessageRule, requires=[HasDice()]):
    async def check(self, message: Message, ctx: Context) -> bool:
        return message.dice.unwrap().emoji == DiceEmoji.FOOTBALL


class IsSlotMachineDice(MessageRule, requires=[HasDice()]):
    async def check(self, message: Message, ctx: Context) -> bool:
        return message.dice.unwrap().emoji == DiceEmoji.SLOT_MACHINE


class IsBowlingDice(MessageRule, requires=[HasDice()]):
    async def check(self, message: Message, ctx: Context) -> bool:
        return message.dice.unwrap().emoji == DiceEmoji.BOWLING


__all__ = (
    "IsBasketballDice",
    "IsBot",
    "IsBowlingDice",
    "IsChat",
    "IsChatId",
    "IsDartDice",
    "IsDice",
    "IsForum",
    "IsGroup",
    "IsLanguageCode",
    "IsPremium",
    "IsPrivate",
    "IsReply",
    "IsSuperGroup",
    "IsUser",
    "IsUserId",
)



import typing

from telegrinder.api.abc import ABCAPI
from telegrinder.bot.cute_types import MessageCute
from telegrinder.bot.dispatch.context import Context
from telegrinder.bot.dispatch.process import check_rule
from telegrinder.bot.rules.abc import ABCRule
from telegrinder.modules import logger
from telegrinder.msgspec_utils import Nothing
from telegrinder.types.objects import Update

from .abc import ABCHandler


class MessageReplyHandler(ABCHandler[MessageCute]):
    def __init__(
        self,
        text: str,
        *rules: ABCRule[MessageCute],
        as_reply: bool = False,
        is_blocking: bool = True,
    ):
        self.text = text
        self.rules = list(rules)
        self.as_reply = as_reply
        self.is_blocking = is_blocking
        self.dataclass = MessageCute
        self.ctx = Context()

    async def check(self, api: ABCAPI, event: Update, ctx: Context | None = None) -> bool:
        ctx = ctx or Context()
        self.ctx |= ctx
        for rule in self.rules:
            if not await check_rule(api, rule, event, self.ctx):
                logger.debug("Rule {!r} failed!", rule)
                return False
        return True

    async def run(self, event: MessageCute) -> typing.Any:
        await event.answer(
            text=self.text,
            reply_to_message_id=(event.message_id if self.as_reply else Nothing),
        )


__all__ = ("MessageReplyHandler",)



import difflib

from telegrinder.bot.dispatch.context import Context

from .abc import Message
from .text import TextMessageRule


class FuzzyText(TextMessageRule):
    def __init__(self, texts: str | list[str], min_ratio: float = 0.7):
        if isinstance(texts, str):
            texts = [texts]
        self.texts = texts
        self.min_ratio = min_ratio

    async def check(self, message: Message, ctx: Context) -> bool:
        match = max(
            difflib.SequenceMatcher(a=message.text.unwrap(), b=text).ratio()
            for text in self.texts
        )
        if match < self.min_ratio:
            return False
        ctx.fuzzy_ratio = match
        return True


__all__ = ("FuzzyText",)



from telegrinder.bot.cute_types import CallbackQueryCute
from telegrinder.bot.dispatch.context import Context

from .abc import BaseReturnManager, register_manager


class CallbackQueryReturnManager(BaseReturnManager[CallbackQueryCute]):
    @register_manager(str)
    @staticmethod
    async def str_manager(value: str, event: CallbackQueryCute, ctx: Context) -> None:
        await event.answer(value)
    
    @register_manager(dict)
    @staticmethod
    async def dict_manager(value: dict, event: CallbackQueryCute, ctx: Context) -> None:
        await event.answer(**value)


__all__ = ("CallbackQueryReturnManager",)



import asyncio
import contextlib
import dataclasses
import typing

from telegrinder.modules import logger

from .abc import ABCLoopWrapper, CoroutineFunc, CoroutineTask


@dataclasses.dataclass
class DelayedTask:
    handler: CoroutineFunc
    seconds: float
    repeat: bool = dataclasses.field(default=False, kw_only=True)
    _cancelled: bool = dataclasses.field(default=False, init=False, repr=False)

    @property
    def is_cancelled(self) -> bool:
        return self._cancelled

    async def __call__(self, *args, **kwargs) -> None:
        while not self.is_cancelled:
            await asyncio.sleep(self.seconds)
            if self.is_cancelled:
                break
            await self.handler(*args, **kwargs)
            if not self.repeat:
                break

    def cancel(self) -> None:
        if not self._cancelled:
            self._cancelled = True


class LoopWrapper(ABCLoopWrapper):
    def __init__(self, tasks: list[CoroutineTask] | None = None):
        self.on_startup: list[CoroutineTask] = []
        self.on_shutdown: list[CoroutineTask] = []
        self.tasks = tasks or []
        self._loop = asyncio.new_event_loop()
        
    def run_event_loop(self) -> None:
        if not self.tasks:
            logger.warning("You run loop with 0 tasks!")

        for startup_task in self.on_startup:
            self._loop.run_until_complete(startup_task)
        for task in self.tasks:
            self._loop.create_task(task)
        
        self.tasks.clear()
        tasks = asyncio.all_tasks(self._loop)
        try:
            while tasks:
                tasks_results, _ = self._loop.run_until_complete(
                    asyncio.wait(tasks, return_when=asyncio.FIRST_EXCEPTION)
                )
                for task_result in tasks_results:
                    try:
                        task_result.result()
                    except BaseException as ex:
                        logger.exception(ex)
                tasks = asyncio.all_tasks(self._loop)
        except KeyboardInterrupt:
            print()  # blank print for ^C
            logger.info("KeyboardInterrupt")
            self.complete_tasks(tasks)
        finally:
            for shutdown_task in self.on_shutdown:
                self._loop.run_until_complete(shutdown_task)
            if self._loop.is_running():
                self._loop.close()
        
    def add_task(self, task: CoroutineFunc | CoroutineTask | DelayedTask):
        if asyncio.iscoroutinefunction(task) or isinstance(task, DelayedTask):
            task = task()
        elif not asyncio.iscoroutine(task):
            raise TypeError("Task should be coroutine or coroutine function.")

        if self._loop and self._loop.is_running():
            self._loop.create_task(task)
        else:
            self.tasks.append(task)
    
    def complete_tasks(self, tasks: set[asyncio.Task[typing.Any]]) -> None:
        tasks = tasks | asyncio.all_tasks(self._loop)
        task_to_cancel = asyncio.gather(*tasks, return_exceptions=True)
        task_to_cancel.cancel()
        with contextlib.suppress(asyncio.CancelledError):
            self._loop.run_until_complete(task_to_cancel)

    def timer(
        self,
        *,
        days: int = 0,
        hours: int = 0,
        minutes: int = 0,
        seconds: float = 0,
    ) -> typing.Callable[[typing.Callable], DelayedTask]:
        seconds += minutes * 60
        seconds += hours * 60 * 60
        seconds += days * 24 * 60 * 60

        def decorator(func: typing.Callable) -> DelayedTask:
            delayed_task = DelayedTask(func, seconds, repeat=False)
            self.add_task(delayed_task)
            return delayed_task

        return decorator

    def interval(
        self,
        *,
        days: int = 0,
        hours: int = 0,
        minutes: int = 0,
        seconds: float = 0,
    ) -> typing.Callable[[typing.Callable], DelayedTask]:
        seconds += minutes * 60
        seconds += hours * 60 * 60
        seconds += days * 24 * 60 * 60

        def decorator(func: typing.Callable) -> DelayedTask:
            delayed_task = DelayedTask(func, seconds, repeat=True)
            self.add_task(delayed_task)
            return delayed_task

        return decorator


__all__ = ("DelayedTask", "LoopWrapper")



import typing

from .base import ComposeError, ScalarNode
from .message import MessageNode


class Text(ScalarNode, str):
    @classmethod
    async def compose(cls, message: MessageNode) -> typing.Self:
        return cls(message.text.expect(ComposeError("Message has no text")))


__all__ = ("Text",)



from telegrinder.bot.cute_types import MessageCute
from telegrinder.bot.dispatch.return_manager import MessageReturnManager

from .abc import BaseStateView


class MessageView(BaseStateView[MessageCute]):
    def __init__(self) -> None:
        self.auto_rules = []
        self.handlers = []
        self.middlewares = []
        self.return_manager = MessageReturnManager()

    def get_state_key(self, event: MessageCute) -> int | None:
        return event.chat.id


__all__ = ("MessageView",)



from telegrinder.bot.dispatch.context import Context
from telegrinder.tools.i18n.base import ABCTranslator

from .abc import ABC, Message, MessageRule, with_caching_translations


class HasText(MessageRule):
    async def check(self, message: Message, ctx: Context) -> bool:
        return bool(message.text)


class TextMessageRule(MessageRule, ABC, requires=[HasText()]):
    pass


class Text(TextMessageRule):
    def __init__(self, texts: str | list[str], ignore_case: bool = False):
        if not isinstance(texts, list):
            texts = [texts]
        self.texts = texts if not ignore_case else list(map(str.lower, texts))
        self.ignore_case = ignore_case

    async def check(self, message: Message, ctx: Context) -> bool:
        text = message.text.unwrap()
        return (text if not self.ignore_case else text.lower()) in self.texts

    @with_caching_translations
    async def translate(self, translator: ABCTranslator) -> "Text":
        return Text(
            texts=[translator.get(text) for text in self.texts],
            ignore_case=self.ignore_case,
        )


__all__ = ("HasText", "Text", "TextMessageRule")



import asyncio
import datetime
import typing

from telegrinder.api.abc import ABCAPI
from telegrinder.bot.dispatch.context import Context
from telegrinder.bot.rules.abc import ABCRule

from .middleware import WaiterMiddleware
from .short_state import Behaviour, EventModel, ShortState

Identificator: typing.TypeAlias = str | int
Storage: typing.TypeAlias = dict[str, dict[Identificator, "ShortState"]]

if typing.TYPE_CHECKING:
    from telegrinder.bot.dispatch.view.abc import ABCStateView, BaseStateView


class WaiterMachine:
    def __init__(self) -> None:
        self.storage: Storage = {}

    async def drop(
        self,
        state_view: "ABCStateView[EventModel]",
        id: Identificator,
        **context,
    ) -> None:
        view_name = state_view.__class__.__name__
        if view_name not in self.storage:
            raise LookupError("No record of view {!r} found".format(view_name))

        short_state = self.storage[view_name].pop(id, None)
        if not short_state:
            raise LookupError(
                "Waiter with identificator {} is not found for view {!r}".format(
                    id,
                    view_name,
                )
            )

        waiters = typing.cast(
            typing.Iterable[asyncio.Future[typing.Any]],
            short_state.event._waiters  # type: ignore
        )
        for future in waiters:
            future.cancel()

        await self.call_behaviour(
            state_view,
            short_state.on_drop_behaviour,
            short_state.event,
            **context,
        )

    async def wait(
        self,
        state_view: "BaseStateView[EventModel]",
        linked: EventModel | tuple[ABCAPI, Identificator],
        *rules: ABCRule[EventModel],
        default: Behaviour = None,
        on_drop: Behaviour = None,
        expiration: datetime.timedelta | int | None = None,
    ) -> tuple[EventModel, Context]:
        if isinstance(expiration, int):
            expiration = datetime.timedelta(seconds=expiration)

        api: ABCAPI
        key: Identificator
        event = asyncio.Event()
        if isinstance(linked, tuple):
            api, key = linked
        else:
            api, key = linked.ctx_api, state_view.get_state_key(linked)  # type: ignore
            if not key:
                raise RuntimeError("Unable to get state key.")

        short_state = ShortState(
            key,
            ctx_api=api,
            event=event,
            rules=rules,
            expiration=expiration,
            default_behaviour=default,
            on_drop_behaviour=on_drop,
        )

        view_name = state_view.__class__.__name__
        if view_name not in self.storage:
            state_view.middlewares.insert(0, WaiterMiddleware(self, state_view))
            self.storage[view_name] = {}

        self.storage[view_name][key] = short_state

        await event.wait()

        e, ctx = getattr(event, "context")
        self.storage[view_name].pop(key)

        return e, ctx

    async def call_behaviour(
        self,
        view: "ABCStateView[EventModel]",
        behaviour: Behaviour,
        event: asyncio.Event | EventModel,
        **context,
    ) -> None:
        if behaviour is None:
            return
        # TODO: add behaviour check
        # TODO: support view as a behaviour
        await behaviour.run(event)


__all__ = ("WaiterMachine",)



import typing_extensions as typing

from telegrinder.api.abc import ABCAPI
from telegrinder.bot.cute_types import BaseCute
from telegrinder.bot.dispatch.context import Context
from telegrinder.bot.dispatch.process import check_rule
from telegrinder.modules import logger
from telegrinder.tools.error_handler import ABCErrorHandler, ErrorHandler
from telegrinder.types import Update

from .abc import ABCHandler

if typing.TYPE_CHECKING:
    from telegrinder.bot.rules import ABCRule

F = typing.TypeVar("F", bound=typing.Callable[typing.Concatenate[typing.Any, ...], typing.Awaitable])
EventT = typing.TypeVar("EventT", bound=BaseCute)
ErrorHandlerT = typing.TypeVar("ErrorHandlerT", bound=ABCErrorHandler, default=ErrorHandler)


class FuncHandler(ABCHandler[EventT], typing.Generic[EventT, F, ErrorHandlerT]):
    def __init__(
        self,
        func: F,
        rules: list["ABCRule[EventT]"],
        is_blocking: bool = True,
        dataclass: type[typing.Any] | None = dict,
        error_handler: ErrorHandlerT | None = None,
    ):
        self.func = func
        self.is_blocking = is_blocking
        self.rules = rules
        self.dataclass = dataclass
        self.error_handler: ErrorHandlerT = error_handler or ErrorHandler()  # type: ignore
        self.ctx = Context()
    
    @property
    def on_error(self):
        return self.error_handler.register_catcher
    
    async def check(self, api: ABCAPI, event: Update, ctx: Context | None = None) -> bool:
        ctx = ctx or Context()
        preset_ctx = self.ctx.copy()
        self.ctx |= ctx
        for rule in self.rules:
            if not await check_rule(api, rule, event, self.ctx):
                logger.debug("Rule {!r} failed!", rule)
                self.ctx = preset_ctx
                return False
        return True

    async def run(self, event: EventT) -> typing.Any:
        if self.dataclass is not None:
            event = self.dataclass(**event.to_dict())
        return (await self.error_handler.run(self.func, event, event.api, self.ctx)).unwrap()


__all__ = ("FuncHandler",)



from telegrinder.bot.cute_types import MessageCute
from telegrinder.bot.dispatch.context import Context

from .abc import BaseReturnManager, register_manager


class MessageReturnManager(BaseReturnManager[MessageCute]):  
    @register_manager(str)
    @staticmethod
    async def str_manager(value: str, event: MessageCute, ctx: Context) -> None:
        await event.answer(value)
    
    @register_manager(list | tuple)
    @staticmethod
    async def seq_manager(value: list[str] | tuple[str, ...], event: MessageCute, ctx: Context) -> None:
        for message in value:
            await event.answer(message)
    
    @register_manager(dict)
    @staticmethod
    async def dict_manager(value: dict, event: MessageCute, ctx: Context) -> None:
        await event.answer(**value)


__all__ = ("MessageReturnManager",)



import typing
from abc import ABC, abstractmethod

CoroutineTask = typing.Coroutine[typing.Any, typing.Any, typing.Any]
CoroutineFunc = typing.Callable[..., CoroutineTask]


class ABCLoopWrapper(ABC):
    @abstractmethod
    def add_task(self, task: CoroutineFunc | CoroutineTask) -> None:
        ...

    @abstractmethod
    def run_event_loop(self) -> None:
        ...


__all__ = ("ABCLoopWrapper",)



import asyncio
import datetime
import typing

from telegrinder.api.abc import ABCAPI
from telegrinder.bot.cute_types import BaseCute
from telegrinder.bot.dispatch.handler.abc import ABCHandler
from telegrinder.bot.rules.abc import ABCRule

if typing.TYPE_CHECKING:
    from .machine import Identificator

EventModel = typing.TypeVar("EventModel", bound=BaseCute)
Behaviour = ABCHandler | None


class ShortState(typing.Generic[EventModel]):
    def __init__(
        self,
        key: "Identificator",
        ctx_api: ABCAPI,
        event: asyncio.Event,
        rules: tuple[ABCRule[EventModel], ...],
        expiration: datetime.timedelta | None = None,
        default_behaviour: Behaviour | None = None,
        on_drop_behaviour: Behaviour | None = None,
    ) -> None:
        self.key = key
        self.ctx_api = ctx_api
        self.event = event
        self.rules = rules
        self.default_behaviour = default_behaviour
        self.expiration = (datetime.datetime.now() + expiration) if expiration else None
        self.on_drop_behaviour = on_drop_behaviour


__all__ = ("ShortState",)



import typing

from fntypes.result import Error, Ok, Result

from telegrinder.api.abc import ABCAPI
from telegrinder.bot.cute_types import BaseCute
from telegrinder.bot.rules.adapter.abc import ABCAdapter
from telegrinder.bot.rules.adapter.errors import AdapterError
from telegrinder.msgspec_utils import Nothing
from telegrinder.types.objects import Model, Update

EventT = typing.TypeVar("EventT", bound=Model)
CuteT = typing.TypeVar("CuteT", bound=BaseCute)


class EventAdapter(ABCAdapter[Update, CuteT]):
    def __init__(self, event_name: str, model: type[CuteT]) -> None:
        self.event_name = event_name
        self.model = model
    
    def __repr__(self) -> str:
        raw_update_type = Update.__annotations__.get(self.event_name, "Unknown")
        raw_update_type = (
            typing.get_args(raw_update_type)[0].__forward_arg__
            if typing.get_args(raw_update_type)
            else raw_update_type
        )
        return "<{}: adapt {} -> {}>".format(
            self.__class__.__name__,
            raw_update_type,
            self.model.__name__,
        )

    async def adapt(self, api: ABCAPI, update: Update) -> Result[CuteT, AdapterError]:
        update_dct = update.to_dict()
        if self.event_name not in update_dct:
            return Error(
                AdapterError(f"Update is not of event type {self.event_name!r}."),
            )
        if update_dct[self.event_name] is Nothing:
            return Error(
                AdapterError(f"Update is not an {self.event_name!r}."),
            )
        return Ok(
            self.model.from_update(update_dct[self.event_name].unwrap(), bound_api=api),
        )


__all__ = ("EventAdapter",)



import typing

from .msgspec_utils import decoder, encoder


def loads(s: str | bytes) -> dict[str, typing.Any] | list[typing.Any]:
    return decoder.decode(s, type=dict[str, typing.Any] | list[typing.Any])  # type: ignore


def dumps(o: dict[str, typing.Any] | list[typing.Any]) -> str:
    return encoder.encode(o)


__all__ = ("dumps", "loads")



import dataclasses

import typing_extensions as typing

from .abc import ABCView
from .callback_query import CallbackQueryView
from .inline_query import InlineQueryView
from .message import MessageView

CallbackQueryViewT = typing.TypeVar(
    "CallbackQueryViewT", bound=ABCView, default=CallbackQueryView
)
InlineQueryViewT = typing.TypeVar(
    "InlineQueryViewT", bound=ABCView, default=InlineQueryView
)
MessageViewT = typing.TypeVar("MessageViewT", bound=ABCView, default=MessageView)


@dataclasses.dataclass(kw_only=True)
class ViewBox(typing.Generic[CallbackQueryViewT, InlineQueryViewT, MessageViewT]):
    callback_query: CallbackQueryViewT = dataclasses.field(  # type: ignore
        default_factory=lambda: CallbackQueryView(),
    )
    inline_query: InlineQueryViewT = dataclasses.field(  # type: ignore
        default_factory=lambda: InlineQueryView(),
    )
    message: MessageViewT = dataclasses.field(  # type: ignore
        default_factory=lambda: MessageView(),
    )

    def get_views(self) -> dict[str, ABCView]:
        return {
            name: view for name, view in self.__dict__.items()
            if isinstance(view, ABCView)
        }


__all__ = ("ViewBox",)



import typing

from .api import ABCAPI, API, APIError, APIResponse, Token
from .bot import (
    ABCDispatch,
    ABCHandler,
    ABCMiddleware,
    ABCPolling,
    ABCReturnManager,
    ABCRule,
    ABCScenario,
    ABCStateView,
    ABCView,
    BaseCute,
    BaseReturnManager,
    BaseStateView,
    BaseView,
    CallbackQueryCute,
    CallbackQueryReturnManager,
    CallbackQueryView,
    Checkbox,
    Dispatch,
    FuncHandler,
    InlineQueryCute,
    InlineQueryReturnManager,
    MessageCute,
    MessageReplyHandler,
    MessageReturnManager,
    MessageRule,
    MessageView,
    Polling,
    SingleChoice,
    Telegrinder,
    ViewBox,
    WaiterMachine,
    register_manager,
)
from .client import ABCClient, AiohttpClient
from .model import Model
from .modules import logger
from .tools import (
    ABCErrorHandler,
    ABCGlobalContext,
    ABCLoopWrapper,
    ABCTranslator,
    ABCTranslatorMiddleware,
    AnyMarkup,
    Button,
    CtxVar,
    DelayedTask,
    ErrorHandler,
    FormatString,
    GlobalContext,
    HTMLFormatter,
    I18nEnum,
    InlineButton,
    InlineKeyboard,
    Keyboard,
    KeyboardSetBase,
    KeyboardSetYAML,
    LoopWrapper,
    ParseMode,
    RowButtons,
    SimpleI18n,
    SimpleTranslator,
    ctx_var,
    magic_bundle,
)

Message: typing.TypeAlias = MessageCute
CallbackQuery: typing.TypeAlias = CallbackQueryCute
InlineQuery: typing.TypeAlias = InlineQueryCute
Bot: typing.TypeAlias = Telegrinder


__all__ = (
    "ABCAPI",
    "ABCClient",
    "ABCDispatch",
    "ABCErrorHandler",
    "ABCGlobalContext",
    "ABCHandler",
    "ABCLoopWrapper",
    "ABCMiddleware",
    "ABCPolling",
    "ABCReturnManager",
    "ABCRule",
    "ABCScenario",
    "ABCStateView",
    "ABCTranslator",
    "ABCTranslatorMiddleware",
    "ABCView",
    "API",
    "APIError",
    "APIResponse",
    "AiohttpClient",
    "AnyMarkup",
    "BaseCute",
    "BaseReturnManager",
    "BaseStateView",
    "BaseView",
    "Bot",
    "Button",
    "CallbackQuery",
    "CallbackQueryCute",
    "CallbackQueryReturnManager",
    "CallbackQueryView",
    "Checkbox",
    "CtxVar",
    "DelayedTask",
    "Dispatch",
    "ErrorHandler",
    "FormatString",
    "FuncHandler",
    "GlobalContext",
    "HTMLFormatter",
    "I18nEnum",
    "InlineButton",
    "InlineKeyboard",
    "InlineQuery",
    "InlineQueryCute",
    "InlineQueryReturnManager",
    "Keyboard",
    "KeyboardSetBase",
    "KeyboardSetYAML",
    "LoopWrapper",
    "Message",
    "MessageCute",
    "MessageReplyHandler",
    "MessageReturnManager",
    "MessageRule",
    "MessageView",
    "Model",
    "ParseMode",
    "Polling",
    "RowButtons",
    "SimpleI18n",
    "SimpleTranslator",
    "SingleChoice",
    "Telegrinder",
    "Token",
    "ViewBox",
    "WaiterMachine",
    "ctx_var",
    "logger",
    "magic_bundle",
    "register_manager",
)
